﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Optimization;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Hive.Experiment {
  [StorableClass]
  public class OptimizerJob : IJob {

    [Storable]
    private IOptimizer optimizer;
    public IOptimizer Optimizer {
      get { return optimizer;  }
      set {
        if (optimizer != null && value != optimizer) {
          DeregisterEvents();
        }
        optimizer = value;
        if (optimizer != null) {
          RegisterEvents();
        }
      }
    }

    [Storable]
    private double progress = 0.0;

    public OptimizerJob() {
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEvents();
    }

    #region IJob Members

    public long JobId {
      get;
      set;
    }

    public double Progress {
      get { return optimizer.ExecutionState == ExecutionState.Stopped ? 1.0 : this.progress; }
    }

    public ExecutionState ExecutionState {
      get { return optimizer.ExecutionState; }
    }

    public void Run() {
      throw new NotSupportedException();
    }

    public void Prepare() {
      optimizer.Prepare();
    }

    public void Start() {
      optimizer.Start();
    }

    public void Stop() {
      optimizer.Stop();
    }

    public event EventHandler JobStopped;

    public event EventHandler JobFailed;

    #endregion

    #region Events
    private void RegisterEvents() {
      optimizer.Stopped += new EventHandler(optimizer_Stopped);
      optimizer.ExceptionOccurred += new EventHandler<Common.EventArgs<Exception>>(optimizer_ExceptionOccurred);
    }

    private void DeregisterEvents() {
      optimizer.Stopped -= new EventHandler(optimizer_Stopped);
      optimizer.ExceptionOccurred -= new EventHandler<Common.EventArgs<Exception>>(optimizer_ExceptionOccurred);
    }

    void optimizer_ExceptionOccurred(object sender, Common.EventArgs<Exception> e) {
      if (optimizer.ExecutionState != ExecutionState.Stopped) this.progress = 0.0;
      else this.progress = 1.0;
      EventHandler handler = JobFailed;
      if (handler != null) handler(this, e);
    }

    void optimizer_Stopped(object sender, EventArgs e) {
      if (optimizer.ExecutionState != ExecutionState.Stopped) this.progress = 0.0;
      else this.progress = 1.0;
      EventHandler handler = JobStopped;
      if (handler != null) handler(this, e);
    }
    #endregion


  }
}
