﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using System.Timers;
using HeuristicLab.Hive.Contracts;
using System.Collections;
using System.Threading;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.ResponseObjects;

namespace HeuristicLab.Hive.Server.ServerConsole {
  public class JobDataFetcher {
    private List<JobState> pollStates;

    public IEnumerable<JobState> PollStates {
      get { return pollStates; }
    }

    public event EventHandler NewDataAvailable;

    private System.Timers.Timer timer;

    private static object locker = new object();

    public List<JobDto> CachedJobs { get; set; }

    private IJobManager jobManager;

    public double Interval {
      get { return timer.Interval; }
      set { timer.Interval = value; }
    }

    private int _nrOfEntriesOnPage;
    public int NrOfEntriesOnPage {
      get {
        return _nrOfEntriesOnPage;
      }
      set {
        Console.WriteLine("Setting for " + PollStates + " changed to: " + value);
        _nrOfEntriesOnPage = value;
        Thread t = new Thread(new ThreadStart(DoUpdateRun));
        t.Start();
      }
    }
    public int CurrentPage { get; set; }

    public JobDataFetcher(IEnumerable<JobState> states, int nrOfEntries) {
      pollStates = new List<JobState>();
      CachedJobs = new List<JobDto>();
      NrOfEntriesOnPage = nrOfEntries;
      CurrentPage = 0;
      pollStates.AddRange(states);
      jobManager = ServiceLocator.GetJobManager();
      timer = new System.Timers.Timer();
      timer.Interval = 5000;
      timer.Elapsed += new ElapsedEventHandler(Timer_Elapsed);
    }

    public void Start() {
      timer.Start();
    }

    public void Forward() {
      CurrentPage++;
      DoUpdateRun();
    }
    public void Backward() {
      if (CurrentPage != 0)
        CurrentPage--;
      DoUpdateRun();
    }

    private void Timer_Elapsed(object sender, ElapsedEventArgs e) {
      DoUpdateRun();
    }

    private void DoUpdateRun() {
      // [chn] Exception occured here: PollStates enumeration changed!
      // todo: make access to PollStates thread-safe!
      // * added lock on locker -> didn't help
      // * made public property PollStates an IEnumerable to be immutable -> didn't help
      // * using for loop

      JobState[] pollStatesCopy = PollStates.ToArray();
      for (int i = 0; i < pollStatesCopy.Length; i++) {
        ResponseList<JobDto> resList = jobManager.GetAllJobsWithFilter(pollStatesCopy[i], CurrentPage * NrOfEntriesOnPage, NrOfEntriesOnPage);
        if (resList.StatusMessage == ResponseStatus.Ok) {
          if (resList.List.Count == 0) {
            if (CurrentPage > 0) {
              CurrentPage--;
            }
          } else {
            CachedJobs = new List<JobDto>();
            CachedJobs.AddRange(resList.List);
          }
        }
      }
      if (NewDataAvailable != null) {
        NewDataAvailable(this, new EventArgs());
      }
    }
  }
}