#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.PluginInfrastructure;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.ServiceModel;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using System.Transactions;
using HeuristicLab.Hive.Server.LINQDataAccess;
using HeuristicLab.Hive.Server.DataAccess;
using HeuristicLab.Hive.Contracts.ResponseObjects;

namespace HeuristicLab.Hive.Server.Core {
  [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, ConcurrencyMode = ConcurrencyMode.Multiple)]
  public class SlaveFacade : ISlaveFacade {

    public SlaveFacade() {
    }

    private ISlaveCommunicator slaveCommunicator = ServiceLocator.GetSlaveCommunicator();

    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    #region ISlaveCommunicator Members

    public Response Login(ClientDto clientInfo) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.Login(clientInfo);
      }
    }

    public ResponseHeartBeat ProcessHeartBeat(HeartBeatData hbData) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.ProcessHeartBeat(hbData);
      }
    }

    public ResponseObject<JobDto> GetJob(Guid clientId) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.GetJob(clientId);
      }
    }

    public ResponseResultReceived StoreFinishedJobResult(Guid clientId, Guid jobId, byte[] result, double percentage, string exception) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.StoreFinishedJobResult(clientId, jobId, result, percentage, exception);
      }
    }

    public Response Logout(Guid clientId) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.Logout(clientId);
      }
    }

    public Response IsJobStillNeeded(Guid jobId) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.IsJobStillNeeded(jobId); 
      }
    }

    public ResponseList<CachedHivePluginInfoDto> GetPlugins(List<HivePluginInfoDto> pluginList) {
      return slaveCommunicator.GetPlugins(pluginList);      
    }

    public ResponseResultReceived ProcessSnapshot(Guid clientId, Guid jobId, byte[] result, double percentage, string exception) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.ProcessSnapshot(clientId, jobId, result, percentage, exception); 
      }
    }

    public ResponseCalendar GetCalendar(Guid clientId) {
      using (contextFactory.GetContext()) {
        return slaveCommunicator.GetCalendar(clientId); 
      }
    }

    public Response SetCalendarStatus(Guid clientId, CalendarState state) { 
      using (contextFactory.GetContext()) {
        return slaveCommunicator.SetCalendarStatus(clientId, state);
      }
    }
    #endregion

    #region IClientFacade Members

    /// <summary>
    /// Do not use automatic transactions here
    /// </summary>
    public Stream GetStreamedJob(Guid clientId) {
      using (contextFactory.GetContext(false)) {
        MultiStream stream = new MultiStream();

        ResponseObject<JobDto> job = null;

        job = ServiceLocator.GetSlaveCommunicator().GetJob(clientId);

        //first send response
        stream.AddStream(new StreamedObject<ResponseObject<JobDto>>(job));

        IInternalJobManager internalJobManager = (IInternalJobManager)ServiceLocator.GetJobManager();

        //second stream the job binary data
        MemoryStream memoryStream = new MemoryStream();
        if (job.Obj != null) {
          stream.AddStream(new MemoryStream(internalJobManager.GetSerializedJobDataById(job.Obj.Id)));
        }

        OperationContext clientContext = OperationContext.Current;
        clientContext.OperationCompleted += new EventHandler(delegate(object sender, EventArgs args) {
          if (stream != null) {
            stream.Dispose();
          }
        });

        return stream;
      }
    }

    public Stream GetStreamedPlugins(List<HivePluginInfoDto> pluginList) {
      return new StreamedObject<ResponseList<CachedHivePluginInfoDto>>(this.GetPlugins(pluginList));
    }

    public ResponseResultReceived StoreFinishedJobResultStreamed(Stream stream) {
      using (contextFactory.GetContext()) {
        return ((IInternalSlaveCommunicator)slaveCommunicator).ProcessJobResult(stream, true);
      }
    }

    public ResponseResultReceived ProcessSnapshotStreamed(Stream stream) {
      using (contextFactory.GetContext()) {
        return ((IInternalSlaveCommunicator)slaveCommunicator).ProcessJobResult(stream, false);  
      }
    }
    #endregion
  }
}
